@@ -2,25 +2,24 @@
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: MIT-0
 """
-import sys
 import argparse
-import logging
+import copy
 import glob
 import json
+import logging
 import os
-import copy
-import six
+import sys
+from pathlib import Path
+from typing import Dict
+
 import jsonschema
+
 import cfnlint.decode.cfn_yaml
-from cfnlint.version import __version__
 from cfnlint.helpers import REGIONS
-try:  # pragma: no cover
-    from pathlib import Path
-except ImportError:  # pragma: no cover
-    from pathlib2 import Path
+from cfnlint.version import __version__
 
 # pylint: disable=too-many-public-methods
-LOGGER = logging.getLogger('cfnlint')
+LOGGER = logging.getLogger("cfnlint")
 
 
 def configure_logging(debug_logging, info_logging):
@@ -33,7 +32,9 @@ def configure_logging(debug_logging, info_logging):
         LOGGER.setLevel(logging.INFO)
     else:
         LOGGER.setLevel(logging.NOTSET)
-    log_formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
+    log_formatter = logging.Formatter(
+        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+    )
     ch.setFormatter(log_formatter)
 
     # make sure all other log handlers are removed before adding it back
@@ -42,12 +43,13 @@ def configure_logging(debug_logging, info_logging):
     LOGGER.addHandler(ch)
 
 
-class ConfigFileArgs(object):
+class ConfigFileArgs:
     """
-        Config File arguments.
-        Parses .cfnlintrc OR .cfnlintrc.yaml OR .cfnlintrc.yml in the Home and Project folder.
+    Config File arguments.
+    Parses .cfnlintrc OR .cfnlintrc.yaml OR .cfnlintrc.yml in the Home and Project folder.
     """
-    file_args = {}
+
+    file_args: Dict = {}
     __user_config_file = None
     __project_config_file = None
     __custom_config_file = None
@@ -56,15 +58,16 @@ class ConfigFileArgs(object):
         # self.file_args = self.get_config_file_defaults()
         self.file_args = {}
         self.default_schema_file = Path(__file__).parent.joinpath(
-            'data/CfnLintCli/config/schema.json')
-        with self.default_schema_file.open() as f:
+            "data/CfnLintCli/config/schema.json"
+        )
+        with self.default_schema_file.open(encoding="utf-8") as f:
             self.default_schema = json.load(f)
         self.schema = self.default_schema if not schema else schema
 
         if config_file:
             self.__custom_config_file = config_file
         else:
-            LOGGER.debug('Looking for CFLINTRC before attempting to load')
+            LOGGER.debug("Looking for CFLINTRC before attempting to load")
             self.__user_config_file, self.__project_config_file = self._find_config()
 
         self.load()
@@ -80,27 +83,24 @@ class ConfigFileArgs(object):
         -------
             > user_config, project_config = self._find_config()
         """
-        config_file_name = '.cfnlintrc'
-        if six.PY34:
-            self.__user_config_file = Path(os.path.expanduser('~')).joinpath(config_file_name)
-        else:
-            self.__user_config_file = Path.home().joinpath(config_file_name)
+        config_file_name = ".cfnlintrc"
+        self.__user_config_file = Path.home().joinpath(config_file_name)
 
         self.__project_config_file = Path.cwd().joinpath(config_file_name)
-        if self._has_file(config_file_name + '.yaml'):
-            self.__project_config_file = Path.cwd().joinpath(config_file_name + '.yaml')
-        elif self._has_file(config_file_name + '.yml'):
-            self.__project_config_file = Path.cwd().joinpath(config_file_name + '.yml')
+        if self._has_file(config_file_name + ".yaml"):
+            self.__project_config_file = Path.cwd().joinpath(config_file_name + ".yaml")
+        elif self._has_file(config_file_name + ".yml"):
+            self.__project_config_file = Path.cwd().joinpath(config_file_name + ".yml")
 
-        user_config_path = ''
-        project_config_path = ''
+        user_config_path = ""
+        project_config_path = ""
 
         if self._has_file(self.__user_config_file):
-            LOGGER.debug('Found User CFNLINTRC')
+            LOGGER.debug("Found User CFNLINTRC")
             user_config_path = self.__user_config_file
 
         if self._has_file(self.__project_config_file):
-            LOGGER.debug('Found Project level CFNLINTRC')
+            LOGGER.debug("Found Project level CFNLINTRC")
             project_config_path = self.__project_config_file
 
         return user_config_path, project_config_path
@@ -128,27 +128,27 @@ class ConfigFileArgs(object):
 
         if self.__custom_config_file:
             custom_config = self._read_config(self.__custom_config_file)
-            LOGGER.debug('Validating Custom CFNLINTRC')
+            LOGGER.debug("Validating Custom CFNLINTRC")
             self.validate_config(custom_config, self.schema)
-            LOGGER.debug('Custom configuration loaded as')
-            LOGGER.debug('%s', custom_config)
+            LOGGER.debug("Custom configuration loaded as")
+            LOGGER.debug("%s", custom_config)
 
             self.file_args = custom_config
         else:
             user_config = self._read_config(self.__user_config_file)
-            LOGGER.debug('Validating User CFNLINTRC')
+            LOGGER.debug("Validating User CFNLINTRC")
             self.validate_config(user_config, self.schema)
 
             project_config = self._read_config(self.__project_config_file)
-            LOGGER.debug('Validating Project CFNLINTRC')
+            LOGGER.debug("Validating Project CFNLINTRC")
             self.validate_config(project_config, self.schema)
 
-            LOGGER.debug('User configuration loaded as')
-            LOGGER.debug('%s', user_config)
-            LOGGER.debug('Project configuration loaded as')
-            LOGGER.debug('%s', project_config)
+            LOGGER.debug("User configuration loaded as")
+            LOGGER.debug("%s", user_config)
+            LOGGER.debug("Project configuration loaded as")
+            LOGGER.debug("%s", project_config)
 
-            LOGGER.debug('Merging configurations...')
+            LOGGER.debug("Merging configurations...")
             self.file_args = self.merge_config(user_config, project_config)
 
     def validate_config(self, config, schema):
@@ -164,12 +164,12 @@ class ConfigFileArgs(object):
         jsonschema.exceptions.ValidationError
             Returned when cfnlintrc doesn't match schema provided
         """
-        LOGGER.debug('Validating CFNLINTRC config with given JSONSchema')
-        LOGGER.debug('Schema used: %s', schema)
-        LOGGER.debug('Config used: %s', config)
+        LOGGER.debug("Validating CFNLINTRC config with given JSONSchema")
+        LOGGER.debug("Schema used: %s", schema)
+        LOGGER.debug("Config used: %s", config)
 
         jsonschema.validate(config, schema)
-        LOGGER.debug('CFNLINTRC looks valid!')
+        LOGGER.debug("CFNLINTRC looks valid!")
 
     def merge_config(self, user_config, project_config):
         """Merge project and user configuration into a single dictionary
@@ -194,12 +194,17 @@ class ConfigFileArgs(object):
         for key in user_config:
             if key in project_config:
                 # If both keys are the same, let's check whether they have nested keys
-                if isinstance(user_config[key], dict) and isinstance(project_config[key], dict):
+                if isinstance(user_config[key], dict) and isinstance(
+                    project_config[key], dict
+                ):
                     self.merge_config(user_config[key], project_config[key])
                 else:
                     user_config[key] = project_config[key]
                     LOGGER.debug(
-                        'Overriding User\'s key %s with Project\'s specific value %s.', key, project_config[key])
+                        "Overriding User's key %s with Project's specific value %s.",
+                        key,
+                        project_config[key],
+                    )
 
         # Project may have unique config we need to copy over too
         # so that we can have user+project config available as one
@@ -220,7 +225,7 @@ class ConfigFileArgs(object):
         config_template = None
 
         if self._has_file(config):
-            LOGGER.debug('Parsing CFNLINTRC')
+            LOGGER.debug("Parsing CFNLINTRC")
             config_template = cfnlint.decode.cfn_yaml.load(str(config))
 
         if not config_template:
@@ -230,8 +235,8 @@ class ConfigFileArgs(object):
 
 
 def comma_separated_arg(string):
-    """ Split a comma separated string """
-    return string.split(',')
+    """Split a comma separated string"""
+    return string.split(",")
 
 
 def _ensure_value(namespace, name, value):
@@ -241,11 +246,22 @@ def _ensure_value(namespace, name, value):
 
 
 class RuleConfigurationAction(argparse.Action):
-    """ Override the default Action """
+    """Override the default Action"""
 
-    def __init__(self, option_strings, dest, nargs=None, const=None, default=None,
-                 type=None, choices=None, required=False, help=None, metavar=None):  # pylint: disable=W0622
-        super(RuleConfigurationAction, self).__init__(
+    def __init__(
+        self,
+        option_strings,
+        dest,
+        nargs=None,
+        const=None,
+        default=None,
+        type=None,
+        choices=None,
+        required=False,
+        help=None,
+        metavar=None,
+    ):  # pylint: disable=W0622
+        super().__init__(
             option_strings=option_strings,
             dest=dest,
             nargs=nargs,
@@ -255,16 +271,17 @@ class RuleConfigurationAction(argparse.Action):
             choices=choices,
             required=required,
             help=help,
-            metavar=metavar)
+            metavar=metavar,
+        )
 
     def _parse_rule_configuration(self, string):
-        """ Parse the config rule structure """
+        """Parse the config rule structure"""
         configs = comma_separated_arg(string)
         results = {}
         for config in configs:
-            rule_id = config.split(':')[0]
-            config_name = config.split(':')[1].split('=')[0]
-            config_value = config.split(':')[1].split('=')[1]
+            rule_id = config.split(":")[0]
+            config_name = config.split(":")[1].split("=")[0]
+            config_value = config.split(":")[1].split("=")[1]
             if rule_id not in results:
                 results[rule_id] = {}
             results[rule_id][config_name] = config_value
@@ -288,9 +305,10 @@ class RuleConfigurationAction(argparse.Action):
             parser.exit()
 
 
-class CliArgs(object):
-    """ Base Args class"""
-    cli_args = {}
+class CliArgs:
+    """Base Args class"""
+
+    cli_args: Dict = {}
 
     def __init__(self, cli_args):
         self.parser = self.create_parser()
@@ -298,12 +316,13 @@ class CliArgs(object):
 
     def create_parser(self):
         """Do first round of parsing parameters to set options"""
+
         class ArgumentParser(argparse.ArgumentParser):
-            """ Override Argument Parser so we can control the exit code"""
+            """Override Argument Parser so we can control the exit code"""
 
             def error(self, message):
                 self.print_help(sys.stderr)
-                self.exit(32, '%s: error: %s\n' % (self.prog, message))
+                self.exit(32, f"{self.prog}: error: {message}\n")
 
         class ExtendAction(argparse.Action):
             """Support argument types that are lists and can be specified multiple times."""
@@ -319,127 +338,211 @@ class CliArgs(object):
                 setattr(namespace, self.dest, items)
 
         usage = (
-            '\nBasic: cfn-lint test.yaml\n'
-            'Ignore a rule: cfn-lint -i E3012 -- test.yaml\n'
-            'Configure a rule: cfn-lint -x E3012:strict=false -t test.yaml\n'
-            'Lint all yaml files in a folder: cfn-lint dir/**/*.yaml'
+            "\nBasic: cfn-lint test.yaml\n"
+            "Ignore a rule: cfn-lint -i E3012 -- test.yaml\n"
+            "Configure a rule: cfn-lint -x E3012:strict=false -t test.yaml\n"
+            "Lint all yaml files in a folder: cfn-lint dir/**/*.yaml"
         )
 
-        parser = ArgumentParser(
-            description='CloudFormation Linter',
-            usage=usage)
-        parser.register('action', 'extend', ExtendAction)
+        parser = ArgumentParser(description="CloudFormation Linter", usage=usage)
+        parser.register("action", "extend", ExtendAction)
 
-        standard = parser.add_argument_group('Standard')
-        advanced = parser.add_argument_group('Advanced / Debugging')
+        standard = parser.add_argument_group("Standard")
+        advanced = parser.add_argument_group("Advanced / Debugging")
 
         # Allow the template to be passes as an optional or a positional argument
         standard.add_argument(
-            'templates', metavar='TEMPLATE', nargs='*', help='The CloudFormation template to be linted')
-        standard.add_argument(
-            '-t', '--template', metavar='TEMPLATE', dest='template_alt',
-            help='The CloudFormation template to be linted', nargs='+', default=[], action='extend')
-        standard.add_argument(
-            '-b', '--ignore-bad-template', help='Ignore failures with Bad template',
-            action='store_true'
+            "templates",
+            metavar="TEMPLATE",
+            nargs="*",
+            help="The CloudFormation template to be linted",
         )
         standard.add_argument(
-            '--ignore-templates', dest='ignore_templates',
-            help='Ignore templates', nargs='+', default=[], action='extend'
+            "-t",
+            "--template",
+            metavar="TEMPLATE",
+            dest="template_alt",
+            help="The CloudFormation template to be linted",
+            nargs="+",
+            default=[],
+            action="extend",
+        )
+        standard.add_argument(
+            "-b",
+            "--ignore-bad-template",
+            help="Ignore failures with Bad template",
+            action="store_true",
+        )
+        standard.add_argument(
+            "--ignore-templates",
+            dest="ignore_templates",
+            help="Ignore templates",
+            nargs="+",
+            default=[],
+            action="extend",
         )
         advanced.add_argument(
-            '-D', '--debug', help='Enable debug logging', action='store_true'
+            "-D", "--debug", help="Enable debug logging", action="store_true"
         )
         advanced.add_argument(
-            '-I', '--info', help='Enable information logging', action='store_true'
+            "-I", "--info", help="Enable information logging", action="store_true"
         )
         standard.add_argument(
-            '-f', '--format', help='Output Format', choices=['quiet', 'parseable', 'json', 'junit', 'pretty', 'sarif']
+            "-f",
+            "--format",
+            help="Output Format",
+            choices=["quiet", "parseable", "json", "junit", "pretty", "sarif"],
         )
         standard.add_argument(
-            '-l', '--list-rules', dest='listrules', default=False,
-            action='store_true', help='list all the rules'
+            "-l",
+            "--list-rules",
+            dest="listrules",
+            default=False,
+            action="store_true",
+            help="list all the rules",
         )
         standard.add_argument(
-            '-r', '--regions', dest='regions', nargs='+', default=[],
-            type=comma_separated_arg, action='extend',
-            help='list the regions to validate against.'
+            "-r",
+            "--regions",
+            dest="regions",
+            nargs="+",
+            default=[],
+            type=comma_separated_arg,
+            action="extend",
+            help="list the regions to validate against.",
         )
         advanced.add_argument(
-            '-a', '--append-rules', dest='append_rules', nargs='+', default=[],
-            type=comma_separated_arg, action='extend',
-            help='specify one or more rules directories using '
-                 'one or more --append-rules arguments. '
+            "-a",
+            "--append-rules",
+            dest="append_rules",
+            nargs="+",
+            default=[],
+            type=comma_separated_arg,
+            action="extend",
+            help="specify one or more rules directories using "
+            "one or more --append-rules arguments. ",
         )
         standard.add_argument(
-            '-i', '--ignore-checks', dest='ignore_checks', nargs='+', default=[],
-            type=comma_separated_arg, action='extend',
-            help='only check rules whose id do not match these values'
+            "-i",
+            "--ignore-checks",
+            dest="ignore_checks",
+            nargs="+",
+            default=[],
+            type=comma_separated_arg,
+            action="extend",
+            help="only check rules whose id do not match these values",
         )
         standard.add_argument(
-            '-c', '--include-checks', dest='include_checks', nargs='+', default=[],
-            type=comma_separated_arg, action='extend',
-            help='include rules whose id match these values'
+            "-c",
+            "--include-checks",
+            dest="include_checks",
+            nargs="+",
+            default=[],
+            type=comma_separated_arg,
+            action="extend",
+            help="include rules whose id match these values",
         )
         standard.add_argument(
-            '-m', '--mandatory-checks', dest='mandatory_checks', nargs='+', default=[],
-            type=comma_separated_arg, action='extend',
-            help='always check rules whose id match these values, regardless of template exclusions'
+            "-m",
+            "--mandatory-checks",
+            dest="mandatory_checks",
+            nargs="+",
+            default=[],
+            type=comma_separated_arg,
+            action="extend",
+            help="always check rules whose id match these values, regardless of template exclusions",
         )
         standard.add_argument(
-            '-e', '--include-experimental', help='Include experimental rules', action='store_true'
+            "-e",
+            "--include-experimental",
+            help="Include experimental rules",
+            action="store_true",
         )
         standard.add_argument(
-            '-x', '--configure-rule', dest='configure_rules', nargs='+', default={},
+            "-x",
+            "--configure-rule",
+            dest="configure_rules",
+            nargs="+",
+            default={},
             action=RuleConfigurationAction,
-            help='Provide configuration for a rule. Format RuleId:key=value. Example: E3012:strict=false'
-        )
-        standard.add_argument('--config-file', dest='config_file',
-                              help='Specify the cfnlintrc file to use')
-        standard.add_argument(
-            '-z', '--custom-rules', dest='custom_rules',
-            help='Allows specification of a custom rule file.'
-        )
-        advanced.add_argument(
-            '-o', '--override-spec', dest='override_spec',
-            help='A CloudFormation Spec override file that allows customization'
-        )
-        advanced.add_argument(
-            '-g', '--build-graph', help='Creates a file in the same directory as the template that models the template\'s resources in DOT format', action='store_true'
-        )
-        advanced.add_argument(
-            '-s', '--registry-schemas', help='one or more directories of CloudFormation Registry Schemas', action='extend', type=comma_separated_arg, nargs='+'
+            help="Provide configuration for a rule. Format RuleId:key=value. Example: E3012:strict=false",
         )
         standard.add_argument(
-            '-v', '--version', help='Version of cfn-lint', action='version',
-            version='%(prog)s {version}'.format(version=__version__)
-        )
-        advanced.add_argument(
-            '-u', '--update-specs', help='Update the CloudFormation Specs',
-            action='store_true'
-        )
-        advanced.add_argument(
-            '--update-documentation', help=argparse.SUPPRESS,
-            action='store_true'
-        )
-        advanced.add_argument(
-            '--update-iam-policies', help=argparse.SUPPRESS,
-            action='store_true'
+            "--config-file",
+            dest="config_file",
+            help="Specify the cfnlintrc file to use",
         )
         standard.add_argument(
-            '--output-file', type=str, default=None,
-            help='Writes the output to the specified file, ideal for producing reports'
+            "-z",
+            "--custom-rules",
+            dest="custom_rules",
+            help="Allows specification of a custom rule file.",
+        )
+        advanced.add_argument(
+            "-o",
+            "--override-spec",
+            dest="override_spec",
+            help="A CloudFormation Spec override file that allows customization",
+        )
+        advanced.add_argument(
+            "-g",
+            "--build-graph",
+            help="Creates a file in the same directory as the template that models the template's resources in DOT format",
+            action="store_true",
+        )
+        advanced.add_argument(
+            "-s",
+            "--registry-schemas",
+            help="one or more directories of CloudFormation Registry Schemas",
+            action="extend",
+            type=comma_separated_arg,
+            nargs="+",
         )
         standard.add_argument(
-            '--merge-configs', default=False, action='store_true',
-            help='Merges lists between configuration layers'
+            "-v",
+            "--version",
+            help="Version of cfn-lint",
+            action="version",
+            version=f"%(prog)s {__version__}",
         )
+        advanced.add_argument(
+            "-u",
+            "--update-specs",
+            help="Update the CloudFormation Specs",
+            action="store_true",
+        )
+        advanced.add_argument(
+            "--update-documentation", help=argparse.SUPPRESS, action="store_true"
+        )
+        advanced.add_argument(
+            "--update-iam-policies", help=argparse.SUPPRESS, action="store_true"
+        )
+        standard.add_argument(
+            "--output-file",
+            type=str,
+            default=None,
+            help="Writes the output to the specified file, ideal for producing reports",
+        )
+        standard.add_argument(
+            "--merge-configs",
+            default=False,
+            action="store_true",
+            help="Merges lists between configuration layers",
+        )
+        standard.add_argument(
+            "--non-zero-exit-code",
+            type=str,
+            default="informational",
+            choices=["informational", "warning", "error", "none"],
+            help="Exit code will be non zero from the specified rule class and higher",
+        )
+        advanced.add_argument("--force", help=argparse.SUPPRESS, action="store_true")
 
         return parser
 
 
-class TemplateArgs(object):
-    """ Per Template Args """
+class TemplateArgs:
+    """Per Template Args"""
 
     def __init__(self, template_args):
         self.set_template_args(template_args)
@@ -450,34 +553,38 @@ class TemplateArgs(object):
     def set_template_args(self, template):
         defaults = {}
         if isinstance(template, dict):
-            configs = template.get('Metadata', {}).get('cfn-lint', {}).get('config', {})
+            metadata = template.get("Metadata", {})
+            if metadata:
+                configs = (
+                    template.get("Metadata", {}).get("cfn-lint", {}).get("config", {})
+                )
 
-            if isinstance(configs, dict):
-                for config_name, config_value in configs.items():
-                    if config_name == 'ignore_checks':
-                        if isinstance(config_value, list):
-                            defaults['ignore_checks'] = config_value
-                    if config_name == 'regions':
-                        if isinstance(config_value, list):
-                            defaults['regions'] = config_value
-                    if config_name == 'append_rules':
-                        if isinstance(config_value, list):
-                            defaults['append_rules'] = config_value
-                    if config_name == 'override_spec':
-                        if isinstance(config_value, (six.string_types)):
-                            defaults['override_spec'] = config_value
-                    if config_name == 'custom_rules':
-                        if isinstance(config_value, (six.string_types)):
-                            defaults['custom_rules'] = config_value
-                    if config_name == 'ignore_bad_template':
-                        if isinstance(config_value, bool):
-                            defaults['ignore_bad_template'] = config_value
-                    if config_name == 'include_checks':
-                        if isinstance(config_value, list):
-                            defaults['include_checks'] = config_value
-                    if config_name == 'configure_rules':
-                        if isinstance(config_value, dict):
-                            defaults['configure_rules'] = config_value
+                if isinstance(configs, dict):
+                    for config_name, config_value in configs.items():
+                        if config_name == "ignore_checks":
+                            if isinstance(config_value, list):
+                                defaults["ignore_checks"] = config_value
+                        if config_name == "regions":
+                            if isinstance(config_value, list):
+                                defaults["regions"] = config_value
+                        if config_name == "append_rules":
+                            if isinstance(config_value, list):
+                                defaults["append_rules"] = config_value
+                        if config_name == "override_spec":
+                            if isinstance(config_value, (str)):
+                                defaults["override_spec"] = config_value
+                        if config_name == "custom_rules":
+                            if isinstance(config_value, (str)):
+                                defaults["custom_rules"] = config_value
+                        if config_name == "ignore_bad_template":
+                            if isinstance(config_value, bool):
+                                defaults["ignore_bad_template"] = config_value
+                        if config_name == "include_checks":
+                            if isinstance(config_value, list):
+                                defaults["include_checks"] = config_value
+                        if config_name == "configure_rules":
+                            if isinstance(config_value, dict):
+                                defaults["configure_rules"] = config_value
 
         self._template_args = defaults
 
@@ -485,8 +592,8 @@ class TemplateArgs(object):
 
 
 # pylint: disable=too-many-public-methods
-class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
-    """ Mixin for the Configs """
+class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs):
+    """Mixin for the Configs"""
 
     def __init__(self, cli_args):
         CliArgs.__init__(self, cli_args)
@@ -494,7 +601,8 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
         configure_logging(self.cli_args.debug, self.cli_args.info)
         TemplateArgs.__init__(self, {})
         ConfigFileArgs.__init__(
-            self, config_file=self._get_argument_value('config_file', False, False))
+            self, config_file=self._get_argument_value("config_file", False, False)
+        )
 
     def _get_argument_value(self, arg_name, is_template, is_config_file):
         cli_value = getattr(self.cli_args, arg_name)
@@ -503,12 +611,14 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
 
         # merge list configurations
         # make sure we don't do an infinite loop so skip this check for merge_configs
-        if arg_name != 'merge_configs':
+        if arg_name != "merge_configs":
             if self.merge_configs:
                 # the CLI will always have an empty list when the item is a list
                 # we will use that to evaluate if we need to merge the lists
                 if isinstance(cli_value, list):
-                    result = cli_value
+                    # Use a copy here, otherwise we will accumulate template level config
+                    # into the cli_value which will  persist between template files
+                    result = cli_value.copy()
                     if isinstance(template_value, list):
                         result.extend(template_value)
                     if isinstance(file_value, list):
@@ -526,46 +636,47 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
 
     @property
     def ignore_checks(self):
-        return self._get_argument_value('ignore_checks', True, True)
+        return self._get_argument_value("ignore_checks", True, True)
 
     @property
     def include_checks(self):
-        results = self._get_argument_value('include_checks', True, True)
-        return ['W', 'E'] + results
+        results = self._get_argument_value("include_checks", True, True)
+        return ["W", "E"] + results
 
     @property
     def mandatory_checks(self):
-        return self._get_argument_value('mandatory_checks', False, True)
+        return self._get_argument_value("mandatory_checks", False, True)
 
     @property
     def include_experimental(self):
-        return self._get_argument_value('include_experimental', True, True)
+        return self._get_argument_value("include_experimental", True, True)
 
     @property
     def regions(self):
-        results = self._get_argument_value('regions', True, True)
+        results = self._get_argument_value("regions", True, True)
         if not results:
-            return ['us-east-1']
-        if 'ALL_REGIONS' in results:
+            default_region_env = os.environ.get("AWS_DEFAULT_REGION", "us-east-1")
+            return [os.environ.get("AWS_REGION", default_region_env)]
+        if "ALL_REGIONS" in results:
             return REGIONS
         return results
 
     @property
     def ignore_bad_template(self):
-        return self._get_argument_value('ignore_bad_template', True, True)
+        return self._get_argument_value("ignore_bad_template", True, True)
 
     @property
     def debug(self):
-        return self._get_argument_value('debug', False, False)
+        return self._get_argument_value("debug", False, False)
 
     @property
     def format(self):
-        return self._get_argument_value('format', False, True)
+        return self._get_argument_value("format", False, True)
 
     @property
     def templates(self):
-        templates_args = self._get_argument_value('templates', False, True)
-        template_alt_args = self._get_argument_value('template_alt', False, False)
+        templates_args = self._get_argument_value("templates", False, True)
+        template_alt_args = self._get_argument_value("template_alt", False, False)
         if template_alt_args:
             filenames = template_alt_args
         elif templates_args:
@@ -574,7 +685,7 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
             return None
 
         # if only one is specified convert it to array
-        if isinstance(filenames, six.string_types):
+        if isinstance(filenames, str):
             filenames = [filenames]
 
         # handle different shells and Config files
@@ -582,11 +693,7 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
         all_filenames = []
         ignore_templates = self._ignore_templates()
         for filename in filenames:
-            if sys.version_info >= (3, 5):
-                # pylint: disable=E1123
-                add_filenames = glob.glob(filename, recursive=True)
-            else:
-                add_filenames = glob.glob(filename)
+            add_filenames = glob.glob(filename, recursive=True)
             # only way to know of the glob failed is to test it
             # then add the filename as requested
             if not add_filenames:
@@ -600,25 +707,21 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
         return sorted(all_filenames)
 
     def _ignore_templates(self):
-        ignore_template_args = self._get_argument_value('ignore_templates', False, True)
+        ignore_template_args = self._get_argument_value("ignore_templates", False, True)
         if ignore_template_args:
             filenames = ignore_template_args
         else:
             return []
 
         # if only one is specified convert it to array
-        if isinstance(filenames, six.string_types):
+        if isinstance(filenames, str):
             filenames = [filenames]
 
         # handle different shells and Config files
         # some shells don't expand * and configparser won't expand wildcards
         all_filenames = []
         for filename in filenames:
-            if sys.version_info >= (3, 5):
-                # pylint: disable=E1123
-                add_filenames = glob.glob(filename, recursive=True)
-            else:
-                add_filenames = glob.glob(filename)
+            add_filenames = glob.glob(filename, recursive=True)
             # only way to know of the glob failed is to test it
             # then add the filename as requested
             if not add_filenames:
@@ -630,53 +733,61 @@ class ConfigMixIn(TemplateArgs, CliArgs, ConfigFileArgs, object):
 
     @property
     def append_rules(self):
-        return self._get_argument_value('append_rules', False, True)
+        return self._get_argument_value("append_rules", False, True)
 
     @property
     def override_spec(self):
-        return self._get_argument_value('override_spec', False, True)
+        return self._get_argument_value("override_spec", False, True)
 
     @property
     def custom_rules(self):
-        """ custom_rules_spec """
-        return self._get_argument_value('custom_rules', False, True)
+        """custom_rules_spec"""
+        return self._get_argument_value("custom_rules", False, True)
 
     @property
     def update_specs(self):
-        return self._get_argument_value('update_specs', False, False)
+        return self._get_argument_value("update_specs", False, False)
 
     @property
     def update_documentation(self):
-        return self._get_argument_value('update_documentation', False, False)
+        return self._get_argument_value("update_documentation", False, False)
 
     @property
     def update_iam_policies(self):
-        return self._get_argument_value('update_iam_policies', False, False)
+        return self._get_argument_value("update_iam_policies", False, False)
 
     @property
     def listrules(self):
-        return self._get_argument_value('listrules', False, False)
+        return self._get_argument_value("listrules", False, False)
 
     @property
     def configure_rules(self):
-        return self._get_argument_value('configure_rules', True, True)
+        return self._get_argument_value("configure_rules", True, True)
 
     @property
     def config_file(self):
-        return self._get_argument_value('config_file', False, False)
+        return self._get_argument_value("config_file", False, False)
 
     @property
     def build_graph(self):
-        return self._get_argument_value('build_graph', False, False)
+        return self._get_argument_value("build_graph", False, False)
 
     @property
     def output_file(self):
-        return self._get_argument_value('output_file', False, True)
+        return self._get_argument_value("output_file", False, True)
 
     @property
     def registry_schemas(self):
-        return self._get_argument_value('registry_schemas', False, True)
+        return self._get_argument_value("registry_schemas", False, True)
 
     @property
     def merge_configs(self):
-        return self._get_argument_value('merge_configs', True, True)
+        return self._get_argument_value("merge_configs", True, True)
+
+    @property
+    def non_zero_exit_code(self):
+        return self._get_argument_value("non_zero_exit_code", False, False)
+
+    @property
+    def force(self):
+        return self._get_argument_value("force", False, False)
